<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../xsl_util.xsl"/>
<xsl:output method="text" encoding="utf-8" indent="yes" />
<xsl:template match="/">
<xsl:variable name="component_name">
  <xsl:call-template name="replace">
    <xsl:with-param name="text" select="nerv_icde/component/name" />
    <xsl:with-param name="replace" select="' '" />
    <xsl:with-param name="by" select="'_'" />
  </xsl:call-template>
</xsl:variable>

#include "Main_window.hpp"
#include "QT/Main_window.hxx"

#include &lt;QMessageBox&gt;
#include &lt;Waregram/_debug.hpp&gt;
#include &lt;Waregram/_thread.hpp&gt;
#include &lt;chrono&gt;
#include &lt;iostream&gt;

namespace <xsl:value-of select="$component_name"/> {

// using namespace std;

const QSize DPSIZE(640, 320);

Main_window::Main_window(The_<xsl:value-of select="$component_name"/>* _cmpt_, QWidget* _parent_)
    : QMainWindow(_parent_), ui_(new Ui::Main_window), cmpt_(_cmpt_)
{
    std::cout &lt;&lt; "Main_window" &lt;&lt; std::endl;

    ui_-&gt;setupUi(this);

    connect(this, SIGNAL(time()), this, SLOT(on_time()));

    // setWindowFlags(Qt::Window | Qt::FramelessWindowHint);

    // this->move(0, 0);

    timer_ = new std::thread(&amp;Main_window::timer, this);
}

Main_window::~Main_window()
{
    wg_delete_join_thread(timer_);
    delete ui_;
}


void Main_window::on_terminate_clicked()   { cmpt_-&gt;terminate(); }
void Main_window::on_finalize_clicked()    { cmpt_-&gt;finalize(); }
void Main_window::on_initialize_clicked()  { cmpt_-&gt;initialize(); }
void Main_window::on_idle_clicked()        { cmpt_-&gt;idle(); }
void Main_window::on_shutdown_clicked()    { cmpt_-&gt;shutdown(); }
void Main_window::on_ready_clicked()       { cmpt_-&gt;ready(); }
void Main_window::on_standby_clicked()     { cmpt_-&gt;standby(); }
void Main_window::on_stop_clicked()        { cmpt_-&gt;stop(); }
void Main_window::on_start_clicked()       { cmpt_-&gt;start(); }
void Main_window::on_run_clicked()         { cmpt_-&gt;run(); }
void Main_window::on_reset_clicked()       { cmpt_-&gt;reset(); }
void Main_window::on_emergency_clicked()   { cmpt_-&gt;emergency(); }
void Main_window::on_release_clicked()     { cmpt_-&gt;release(); }


void Main_window::on_frequency_valueChanged(double _value)
{
    if(_value &gt; 1) { cmpt_-&gt;frequency((int)_value); }
    else {
        cmpt_-&gt;interval((int)(1.0 / _value));
    }
}

void Main_window::on_time()
{
    // std::cout &lt;&lt; "on_time : " &lt;&lt; std::endl;
    std::string state = get_str_of_State(cmpt_->state_->state);
    if(cmpt_->state_->emergency_state) state += "(emergency)";
    ui_-&gt;state_-&gt;setText(state.c_str());

    ui_-&gt;error_-&gt;clear();
  <xsl:for-each select="/nerv_icde/component/proxy_list/row/object_list/row">
    if(cmpt_-&gt;err_<xsl:value-of select="object_name"/>_conn_) {
        ui_-&gt;error_-&gt;appendPlainText(cmpt_-&gt;err_<xsl:value-of select="object_name"/>_conn_);
    }
  </xsl:for-each>
}

void Main_window::timer()
{
    while(timer_) {
        emit time();
        std::this_thread::sleep_for(std::chrono::milliseconds(33));
    }
}

} // namespace <xsl:value-of select="$component_name"/>

</xsl:template>
</xsl:stylesheet>
