<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_stub_class_id" />

<xsl:template match="/">
  <xsl:for-each select="/nerv_icde/component/stub_list/row[id=$export_stub_class_id]">
    <xsl:variable name="method_domain_id" select="method_domain_id" />
    <xsl:variable name="interface_name" select="interface_name" />
    <xsl:variable name="class_name" select="class_name" />

#include "<xsl:value-of select="$class_name"/>.h"
#include &lt;stdio.h&gt;

using namespace Nerv36;

<xsl:value-of select="$class_name"/>::<xsl:value-of select="$class_name"/>()
{
}

<xsl:value-of select="$class_name"/>::~<xsl:value-of select="$class_name"/>()
{
}

Nerv36::Error <xsl:value-of select="$class_name"/>::start_for_python(const char* _name_, int _port_)
{
    Nerv36::Error ret;
    Py_BEGIN_ALLOW_THREADS
    ret = Nerv36::Stub::start_for_python(_name_, _port_);
    Py_END_ALLOW_THREADS
    return ret;
}

Nerv36::Error <xsl:value-of select="$class_name"/>::stop()
{
    Nerv36::Error ret;
    Py_BEGIN_ALLOW_THREADS
    ret = Nerv36::Stub::stop();
    Py_END_ALLOW_THREADS
    return ret;
}

void <xsl:value-of select="$class_name"/>::on_connect(Nerv36::Address _proxy_addr)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_connect")(_proxy_addr);
    PyGILState_Release(state);
}

void <xsl:value-of select="$class_name"/>::on_disconnect(Nerv36::Address _proxy_addr)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_disconnect")(_proxy_addr);
    PyGILState_Release(state);
}

    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">

            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:choose>
              <xsl:when test="type='CALLBACK'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_return(::Nerv36::Msg_num _msg_num, ::Nerv36::Error _err,<xsl:if test="in_data!='(null)'"> const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _ret_dat_,</xsl:if> double _elapsed)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_return")(_msg_num, _err, _ret_dat_, _elapsed);
    PyGILState_Release(state);
}
              </xsl:when>
              <xsl:when test="type='SIGNALBACK'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_return(::Nerv36::Msg_num _msg_num, ::Nerv36::Error _err, double _elapsed)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_return")(_msg_num, _err, _elapsed);
    PyGILState_Release(state);
}
              </xsl:when>
              <xsl:when test="type='CALL'">
                <xsl:choose>
                  <xsl:when test="in_data='(null)' and out_data='(null)'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( ::Nerv36::Address _proxy_addr, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>")(_proxy_addr, _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}
                  </xsl:when>
                  <xsl:when test="in_data='(null)' and out_data!='(null)'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>(::Nerv36::Address _proxy_addr, Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    _ret_dat_->init_dynamic_buffer(65536);
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>")(_proxy_addr, boost::ref(_ret_dat_), _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}
                  </xsl:when>
                  <xsl:when test="in_data!='(null)' and out_data='(null)'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>(::Nerv36::Address _proxy_addr, const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>")(_proxy_addr, boost::ref(_cal_dat_), _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}
                  </xsl:when>
                  <xsl:when test="in_data!='(null)' and out_data!='(null)'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( ::Nerv36::Address _proxy_addr, const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_, Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    _ret_dat_->init_dynamic_buffer(65536);
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>")(_proxy_addr, boost::ref(_cal_dat_), boost::ref(_ret_dat_), _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}
                  </xsl:when>
                </xsl:choose>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_cancel( ::Nerv36::Address _proxy_addr, ::Nerv36::Error _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_cancel")(_proxy_addr, _err);
    PyGILState_Release(state);
}
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>( Address _proxy_addr, <xsl:if test="in_data!='(null)'">const ::Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _elapsed)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>")(_proxy_addr, <xsl:if test="in_data!='(null)'">boost::ref(_sig_dat_),</xsl:if> _elapsed);
    PyGILState_Release(state);
}

void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_cancel(Address _proxy_addr, Error _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_cancel")(_proxy_addr, _err);
    PyGILState_Release(state);
}
              </xsl:when>
              <xsl:when test="type='EVENT'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link( ::Nerv36::Address _proxy_addr, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>_link")(_proxy_addr, _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}

void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(::Nerv36::Address _proxy_addr, Error _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_unlink")(_proxy_addr,_err);
    PyGILState_Release(state);
}
              </xsl:when>
              <xsl:when test="type='COMMAND'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link(::Nerv36::Address _proxy_addr, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>_link")(_proxy_addr, _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}

void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(::Nerv36::Address _proxy_addr, Error _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_unlink")(_proxy_addr,_err);
    PyGILState_Release(state);
}
                <xsl:choose >
                  <xsl:when test="in_data='(null)'">
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync(::Nerv36::Address _proxy_addr, Nerv36::Sync_model _sync_model, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_sync")(_proxy_addr, (Nerv36::uInt8)_sync_model, _elapsed, _limit);
    PyGILState_Release(state);
}
                  </xsl:when>
                  <xsl:otherwise>
void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync(::Nerv36::Address _proxy_addr, Nerv36::Sync_model _sync_model, double _elapsed, double _limit, const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_sync")(_proxy_addr, (Nerv36::uInt8)_sync_model, _elapsed, _limit, boost::ref(_cmd_dat_));
    PyGILState_Release(state);
}
                  </xsl:otherwise>
                </xsl:choose>
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
::Nerv36::Error <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link(::Nerv36::Address _proxy_addr, bool _reliability, double _elapsed, double _limit)
{
    PyGILState_STATE state = PyGILState_Ensure();
    Nerv36::Error ret_err = this->get_override("on_<xsl:value-of select="$method_name"/>_link")(_proxy_addr, _reliability, _elapsed, _limit);
    PyGILState_Release(state);
    return ret_err;
}

void <xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink(::Nerv36::Address _proxy_addr, Error _err)
{
    PyGILState_STATE state = PyGILState_Ensure();
    this->get_override("on_<xsl:value-of select="$method_name"/>_unlink")(_proxy_addr, _err );
    PyGILState_Release(state);
}
              </xsl:when>
              <xsl:when test="type='BROADCAST'">
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
