<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_stub_class_id" />

<xsl:template match="/">
  <xsl:for-each select="/nerv_icde/component/stub_list/row[id=$export_stub_class_id]">
    <xsl:variable name="method_domain_id" select="method_domain_id" />
    <xsl:variable name="interface_name" select="interface_name" />
    <xsl:variable name="class_name" select="class_name" />

    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="$method_domain_id" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>
#pragma once

#include "<xsl:value-of select="$class_name"/>.h"

#include &lt;boost/python.hpp&gt;
namespace py = boost::python;




void export_stub_<xsl:value-of select="$class_name"/>()
{
    py::class_&lt;::<xsl:value-of select="$class_name"/>, boost::noncopyable&gt;("<xsl:value-of select="$class_name"/>")
        .def("start", &amp;::<xsl:value-of select="$class_name"/>::start_for_python)
        .def("stop", &amp;::<xsl:value-of select="$class_name"/>::stop)
        .def("is_started", &amp;::<xsl:value-of select="$class_name"/>::is_started)

    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">

            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>

            <xsl:choose>

              <xsl:when test="type='CALLBACK'">
        // CALLBACK
        // call (이름 자체가 call 함수명)
        .def("<xsl:value-of select="$method_name"/>_delegate_callback", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_delegate_callback)
        .def("<xsl:value-of select="$method_name"/>", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>)
        .def("<xsl:value-of select="$method_name"/>_cancel", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_cancel)
        .def("on_<xsl:value-of select="$method_name"/>_return", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_return)
              </xsl:when>
              <xsl:when test="type='SIGNALBACK'">
        // SIGNALBACK
        // signal (이름 자체가 call 함수명)
        .def("<xsl:value-of select="$method_name"/>_delegate_signalback", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_delegate_signalback)
        .def("<xsl:value-of select="$method_name"/>", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>)
        .def("<xsl:value-of select="$method_name"/>_cancel", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_cancel)
        .def("on_<xsl:value-of select="$method_name"/>_return", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_return)
              </xsl:when>

              <xsl:when test="type='CALL'">
        // CALL
        .def("on_<xsl:value-of select="$method_name"/>", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>)
        .def("on_<xsl:value-of select="$method_name"/>_cancel", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_cancel)
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
        // SIGNAL
        .def("on_<xsl:value-of select="$method_name"/>", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>)
        .def("on_<xsl:value-of select="$method_name"/>_cancel", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_cancel)
              </xsl:when>
              <xsl:when test="type='EVENT'">
        // connection methods
        .def("on_<xsl:value-of select="$method_name"/>_link", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link)
        .def("on_<xsl:value-of select="$method_name"/>_unlink", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink)
        .def("<xsl:value-of select="$method_name"/>_unlink", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_unlink)
        // event
        .def("<xsl:value-of select="$method_name"/>", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>)
              </xsl:when>
              <xsl:when test="type='COMMAND'">
        // connection methods
        .def("on_<xsl:value-of select="$method_name"/>_link", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link)
        .def("on_<xsl:value-of select="$method_name"/>_unlink", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink)
        .def("<xsl:value-of select="$method_name"/>_unlink", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_unlink)
        // on_sync
        .def("on_<xsl:value-of select="$method_name"/>_sync", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_sync)
        <xsl:if test="in_data!='(null)'">
        // read
        .def("<xsl:value-of select="$method_name"/>_read", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_read)
        </xsl:if>
        // set_sync
        .def("<xsl:value-of select="$method_name"/>_set_sync", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_set_sync)
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
        // connection methods
        .def("on_<xsl:value-of select="$method_name"/>_link", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_link)
        .def("on_<xsl:value-of select="$method_name"/>_unlink", &amp;::<xsl:value-of select="$class_name"/>::on_<xsl:value-of select="$method_name"/>_unlink)
        .def("<xsl:value-of select="$method_name"/>_unlink", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_unlink)
        <xsl:if test="out_data!='(null)'">
        // write
        .def("<xsl:value-of select="$method_name"/>_write", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_write)
        // get / post / read
        .def("<xsl:value-of select="$method_name"/>_get", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_get, py::return_value_policy &lt;py::reference_existing_object&gt;())
        .def("<xsl:value-of select="$method_name"/>_post", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_post)
        .def("<xsl:value-of select="$method_name"/>_read", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_read)
        </xsl:if>
              </xsl:when>
              <xsl:when test="type='BROADCAST'">
        .def("<xsl:value-of select="$method_name"/>_broadcast", &amp;::<xsl:value-of select="$class_name"/>::<xsl:value-of select="$method_name"/>_broadcast)
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>

    ;
}


  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
