<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:include href="../../../../../xsl_util.xsl"/>

<xsl:output method="text" encoding="utf-8" indent="yes"/>

<xsl:param name = "export_stub_class_id" />

<xsl:template match="/">
  <xsl:for-each select="/nerv_icde/component/stub_list/row[id=$export_stub_class_id]">
    <xsl:variable name="method_domain_id" select="method_domain_id" />
    <xsl:variable name="interface_name" select="interface_name" />
    <xsl:variable name="class_name" select="class_name" />

    <xsl:variable name="method_domain_path">
      <xsl:call-template name="get_method_domain_path">
        <xsl:with-param name="mathod_domain_id" select="$method_domain_id" />
      </xsl:call-template>
    </xsl:variable>

    <xsl:variable name="method_domain_namespace">
      <xsl:call-template name="replace">
        <xsl:with-param name="text" select="$method_domain_path" />
        <xsl:with-param name="replace" select="'/'" />
        <xsl:with-param name="by" select="'::'" />
      </xsl:call-template>
    </xsl:variable>

#pragma once

#include &lt;Nerv36/Method<xsl:value-of select="$method_domain_path"/>/<xsl:value-of select="method_domain_name"/>.h&gt;

#include &lt;boost/python.hpp&gt;
namespace py = boost::python;



class <xsl:value-of select="$class_name"/> :
    public Nerv36::Method<xsl:value-of select="$method_domain_namespace"/>::stub_interface_<xsl:value-of select="$interface_name"/>, public py::wrapper&lt;::<xsl:value-of select="$class_name"/>&gt;
{
public:
    ::Nerv36::Error  start_for_python(const char* _name_, int _port_);
    ::Nerv36::Error  stop();

protected:
    virtual void on_connect(Nerv36::Address _proxy_addr);
    virtual void on_disconnect(Nerv36::Address _proxy_addr);

public:
    <xsl:value-of select="$class_name"/>();
    ~<xsl:value-of select="$class_name"/>();

    <xsl:for-each select="/nerv_icde/methoddomain/row[id=$method_domain_id]">
      <xsl:for-each select="interface/row[name=$interface_name]">
        <xsl:for-each select="membermethod/row">
          <xsl:variable name="method_name" select="method_name" />
          <xsl:for-each select="../../../../method/row[name=$method_name]">

            <xsl:variable name="in_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="in_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>
            <xsl:variable name="out_data_domain_namespace">
              <xsl:call-template name="get_data_domain_namespace">
                <xsl:with-param name="data_domain_id" select="out_data_domain_id" />
              </xsl:call-template>
            </xsl:variable>

            <xsl:if test="type!='BROADCAST'" >
    /***************************************************************************
        Method       : <xsl:value-of select="name"/>
        Input        : <xsl:if test="in_data!='(null)'"><xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/></xsl:if>
        output       : <xsl:if test="out_data!='(null)'"><xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/></xsl:if>
        Method Type  : <xsl:value-of select="type"/>
        Code         : <xsl:value-of select="code"/>
        Description  : <xsl:value-of select="description"/>
    ***************************************************************************/
            </xsl:if>

            <xsl:choose>
              <xsl:when test="type='CALLBACK'">
    virtual void on_<xsl:value-of select="$method_name"/>_return(::Nerv36::Msg_num _msg_num, ::Nerv36::Error _err,<xsl:if test="in_data!='(null)'"> const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _ret_dat_,</xsl:if> double _elapsed);
              </xsl:when>
              <xsl:when test="type='SIGNALBACK'">
    virtual void on_<xsl:value-of select="$method_name"/>_return(::Nerv36::Msg_num _msg_num, ::Nerv36::Error _err, double _elapsed);
              </xsl:when>
              <xsl:when test="type='CALL'">
    ::Nerv36::Error on_<xsl:value-of select="$method_name"/>( ::Nerv36::Address _proxy_addr, <xsl:if test="in_data!='(null)'">const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cal_dat_, </xsl:if><xsl:if test="out_data!='(null)'">Nerv36::Data<xsl:value-of select="$out_data_domain_namespace"/>::<xsl:value-of select="out_data"/>* _ret_dat_, </xsl:if> double _elapsed, double _limit);
    void on_<xsl:value-of select="$method_name"/>_cancel(::Nerv36::Address _proxy_addr, ::Nerv36::Error _err);
              </xsl:when>
              <xsl:when test="type='SIGNAL'">
    void on_<xsl:value-of select="$method_name"/>( ::Nerv36::Address _proxy_addr, <xsl:if test="in_data!='(null)'">const ::Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _sig_dat_,</xsl:if> double _elapsed);
    void on_<xsl:value-of select="$method_name"/>_cancel( ::Nerv36::Address _proxy_addr, ::Nerv36::Error _err);
              </xsl:when>
              <xsl:when test="type='EVENT'">
    ::Nerv36::Error on_<xsl:value-of select="$method_name"/>_link( ::Nerv36::Address _proxy_addr, double _elapsed, double _limit);
    void on_<xsl:value-of select="$method_name"/>_unlink( ::Nerv36::Address _proxy_addr, ::Nerv36::Error _err);
              </xsl:when>
              <xsl:when test="type='COMMAND'">
    ::Nerv36::Error on_<xsl:value-of select="$method_name"/>_link(::Nerv36::Address _proxy_addr, double _elapsed, double _limit);
    void on_<xsl:value-of select="$method_name"/>_unlink(::Nerv36::Address _proxy_addr, ::Nerv36::Error _err);
    void on_<xsl:value-of select="$method_name"/>_sync(::Nerv36::Address _proxy_addr, Nerv36::Sync_model _sync_model, double _elapsed, double _limit<xsl:if test="in_data!='(null)'">, const Nerv36::Data<xsl:value-of select="$in_data_domain_namespace"/>::<xsl:value-of select="in_data"/>* _cmd_dat_</xsl:if>);
              </xsl:when>
              <xsl:when test="type='INFORMATION'">
    ::Nerv36::Error on_<xsl:value-of select="$method_name"/>_link( ::Nerv36::Address _proxy_addr, bool _reliability, double _elapsed, double _limit);
    void on_<xsl:value-of select="$method_name"/>_unlink(::Nerv36::Address _proxy_addr, ::Nerv36::Error _err);
              </xsl:when>
            </xsl:choose>
          </xsl:for-each>
        </xsl:for-each>
      </xsl:for-each>
    </xsl:for-each>
};


  </xsl:for-each>
</xsl:template>

</xsl:stylesheet>
