<?xml version="1.0" encoding="utf-8"?>
<!-- copyright@ 2020, Waregram , All right Reserved. -->
<xsl:stylesheet version="2.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:template name="update_year">
  <xsl:value-of select="substring-before(/nerv_system_interface/system/update,'-')" />
</xsl:template>


<xsl:template name="update_date">
  <xsl:value-of select="substring-before(/nerv_system_interface/system/update,' ')" />
</xsl:template>

<xsl:template name="create_date">
  <xsl:value-of select="substring-before(/nerv_system_interface/system/create,' ')" />
</xsl:template>


<xsl:template name="replace">
  <xsl:param name="text" />
  <xsl:param name="replace" />
  <xsl:param name="by" />
  <xsl:choose>
    <xsl:when test="contains($text, $replace)">
      <xsl:value-of select="substring-before($text,$replace)" />
      <xsl:value-of select="$by" />
      <xsl:call-template name="replace">
      <xsl:with-param name="text" select="substring-after($text,$replace)" />
      <xsl:with-param name="replace" select="$replace" />
      <xsl:with-param name="by" select="$by" />
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:value-of select="$text" />
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template name="get_data_domain_path">
  <xsl:param name="data_domain_id" />
  <xsl:for-each select="/nerv_icde/datadomain/row">
    <xsl:if test="id=$data_domain_id">
      <xsl:value-of select="path" />
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<xsl:template name="get_data_domain_namespace">
  <xsl:param name="data_domain_id" />
  <xsl:for-each select="/nerv_icde/datadomain/row">
    <xsl:if test="id=$data_domain_id">
      <xsl:variable name="namespace">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="'::'" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="$namespace" />
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template name="get_data_domain_namespace_empty">
  <xsl:param name="data_domain_id" />
  <xsl:for-each select="/nerv_icde/datadomain/row">
    <xsl:if test="id=$data_domain_id">
      <xsl:variable name="namespace">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="''" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="$namespace" />
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template name="get_data_domain_namespace_dot">
  <xsl:param name="data_domain_id" />
  <xsl:for-each select="/nerv_icde/datadomain/row">
    <xsl:if test="id=$data_domain_id">
      <xsl:variable name="namespace">
        <xsl:call-template name="replace">
          <xsl:with-param name="text" select="path" />
          <xsl:with-param name="replace" select="'/'" />
          <xsl:with-param name="by" select="'.'" />
        </xsl:call-template>
      </xsl:variable>
      <xsl:value-of select="$namespace" />
    </xsl:if>
  </xsl:for-each>
</xsl:template>

<xsl:template name="get_original_type">
  <xsl:param name="data_domain_id" />
  <xsl:param name="data" />
  <xsl:for-each select="/nerv_icde/datadomain/row[id=$data_domain_id]">
    <xsl:for-each select="data/row[name=$data]">
      <xsl:if test="type!='alias'">
        <xsl:value-of select="type" />
      </xsl:if>
      <xsl:if test="type!='alias'">
        <xsl:call-template name="get_original_type">
          <xsl:with-param name="data_domain_id" select="real_domain_id" />
          <xsl:with-param name="data" select="real_data" />
        </xsl:call-template>
      </xsl:if>
    </xsl:for-each>
  </xsl:for-each>
</xsl:template>


<xsl:template name="get_method_domain_namespace">
  <xsl:param name="method_domain_id" />
  <xsl:for-each select="/nerv_icde/methoddomain/row">
  <xsl:if test="id=$method_domain_id">
    <xsl:variable name="namespace">
    <xsl:call-template name="replace">
     <xsl:with-param name="text" select="path" />
     <xsl:with-param name="replace" select="'/'" />
     <xsl:with-param name="by" select="'::'" />
    </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$namespace" />
  </xsl:if>
  </xsl:for-each>
</xsl:template>



<xsl:template name="get_method_domain_path">
  <xsl:param name="mathod_domain_id" />
  <xsl:for-each select="/nerv_icde/methoddomain/row">
    <xsl:if test="id=$mathod_domain_id">
      <xsl:value-of select="path" />
    </xsl:if>
  </xsl:for-each>
</xsl:template>


<xsl:template name="space">
  <xsl:param name="count" />
  <xsl:if test="$count &gt; 0">
    <xsl:text><![CDATA[ ]]></xsl:text>
    <xsl:call-template name="space">
      <xsl:with-param name="count" select="($count)-1"/>
    </xsl:call-template>
  </xsl:if>
</xsl:template>


<xsl:template name="remove_spechar">
    <xsl:param name="text" />
    <xsl:variable name="text_1">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text" />
            <xsl:with-param name="replace" select="'~'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_2">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_1" />
            <xsl:with-param name="replace" select="'!'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_3">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_2" />
            <xsl:with-param name="replace" select="'@'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_4">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_3" />
            <xsl:with-param name="replace" select="'#'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_5">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_4" />
            <xsl:with-param name="replace" select="'$'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_6">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_5" />
            <xsl:with-param name="replace" select="'%'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_7">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_6" />
            <xsl:with-param name="replace" select="'^'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_8">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_7" />
            <xsl:with-param name="replace" select="'&amp;'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_9">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_8" />
            <xsl:with-param name="replace" select="'*'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_10">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_9" />
            <xsl:with-param name="replace" select="'('" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_11">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_10" />
            <xsl:with-param name="replace" select="')'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_12">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_11" />
            <xsl:with-param name="replace" select="'-'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_13">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_12" />
            <xsl:with-param name="replace" select="'+'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_14">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_13" />
            <xsl:with-param name="replace" select="'`'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_15">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_14" />
            <xsl:with-param name="replace" select="'='" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_16">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_15" />
            <xsl:with-param name="replace" select="'{'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_17">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_16" />
            <xsl:with-param name="replace" select="'}'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_18">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_17" />
            <xsl:with-param name="replace" select="'['" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_19">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_18" />
            <xsl:with-param name="replace" select="']'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_20">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_19" />
            <xsl:with-param name="replace" select="';'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_21">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_20" />
            <xsl:with-param name="replace" select="':'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_22">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_21" />
            <xsl:with-param name="replace" select="'apos;'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_23">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_22" />
            <xsl:with-param name="replace" select="'&quot;'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_24">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_23" />
            <xsl:with-param name="replace" select="','" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_25">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_24" />
            <xsl:with-param name="replace" select="'.'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_26">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_25" />
            <xsl:with-param name="replace" select="'&lt;'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_27">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_26" />
            <xsl:with-param name="replace" select="'&gt;'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_28">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_27" />
            <xsl:with-param name="replace" select="'/'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_29">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_28" />
            <xsl:with-param name="replace" select="'?'" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:variable name="text_30">
        <xsl:call-template name="replace">
            <xsl:with-param name="text" select="$text_29" />
            <xsl:with-param name="replace" select="' '" />
            <xsl:with-param name="by" select="'_'" />
        </xsl:call-template>
    </xsl:variable>
    <xsl:value-of select="$text_30"/>
</xsl:template>


</xsl:stylesheet>